/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* gbf-basic-project.c
 *
 * Copyright (C) 2002  Timothee Besset
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Timothee Besset <ttimo@ttimo.net>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/wait.h>
#include <gnome.h>
#include <bonobo/bonobo-exception.h>
#include <bonobo/bonobo-property-bag.h>
#include <bonobo/bonobo-event-source.h>
#include <fcntl.h>
#include <libgnome/gnome-macros.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "gbf-basic-project.h"

struct _GbfBasicProjectPrivate {
	char *project_dir;
	/* Build callbacks. */
	GList *callbacks;
};

static GbfProject *parent_class;

enum {
	PROP_0,
	PROP_PROJECT_DIR
};

typedef struct {
	GbfProject *project;
	int num_channels;
	GList *callbacks;
} BuildInfo;

#define UNIMPLEMENTED g_warning(G_STRLOC": unimplemented")

/*
 * GbfProjectIface methods ------------------------------------------
 */

static void
impl_load (GbfProject *project,
	   const char *path,
	   GError    **error)
{
//	GString *file;
	GbfBasicProject *prj = GBF_BASIC_PROJECT (project);

	if (prj->priv->project_dir) {
		g_warning("Project not empty");
		return;	
	}

	if (!g_file_test (path, G_FILE_TEST_IS_DIR)) {
		if (error) {
			*error = g_error_new (GBF_PROJECT_ERROR,
					      GBF_PROJECT_ERROR_DOESNT_EXIST,
					      _("Project doesn't exist"));
		}
		return;
	}

	prj->priv->project_dir = g_strdup(path);	
}

static void
build_msg (BuildInfo      *info,
	   GbfBuildMessage type,
	   gpointer        msg)
{
	GList *l;

	for (l = info->callbacks; l; l = l->next) {
		GbfBasicBuildCallback *cb = l->data;
		(* cb->callback) (GBF_PROJECT (info->project),
				  type, msg,
				  cb->user_data);
	}
}

static gboolean
build_output_cb (GIOChannel  *chan,
		 GIOCondition cond,
		 gpointer     user_data)
{
	BuildInfo *info = user_data;
	char *line;
	gsize len;
	gsize term;
	GError *err = NULL;
	GIOStatus status;

	status = g_io_channel_read_line (chan, &line, &len, &term, &err);
	if (status != G_IO_STATUS_NORMAL || line == NULL || err != NULL) {
		info->num_channels--;
		if (info->num_channels == 0) {
			build_msg (info, GBF_BUILD_END, "Build ended");

			g_signal_emit_by_name (G_OBJECT (info->project),
					       "build_stop", TRUE);

			/* Cleanup. */
			g_free (info);
		}

		return FALSE;
	}

	build_msg (info, GBF_BUILD_OUTPUT, (gpointer)line);
	g_free (line);

	return TRUE;
}

static void
impl_build (GbfProject  *project,
	    GbfBuildType type,
	    GError     **error)
{
	/* TODO: no build queue - just build - ignore all types and stuff */
	/* TODO: gbf-am-build.c has code to parse build output. This needs to be extracted and re-used.. */
	/* TODO: read the build command from .anjuta file */	

	BuildInfo *info;
	GbfBasicProject *prj = GBF_BASIC_PROJECT (project);
	static const char *build_argv[] = { "make", NULL };
	
	info = g_new0 (BuildInfo, 1);
	info->project = project;
	info->callbacks = prj->priv->callbacks;
	info->num_channels = 2;
	
	int output, err, pid;
	GIOChannel *out_channel, *err_channel;
	
	if (!g_spawn_async_with_pipes (prj->priv->project_dir,
				       (char**)build_argv, NULL,
				       0,
				       NULL, NULL,
				       &pid, 
				       NULL, &output, &err,
				       NULL)) {
		g_warning ("Couldn't spawn %s\n", build_argv[0]);
		return;
	}	

	out_channel = g_io_channel_unix_new (output);
	err_channel = g_io_channel_unix_new (err);
	
	g_io_add_watch (out_channel, G_IO_IN | G_IO_ERR | G_IO_HUP, 
			build_output_cb, 
			info);
	g_io_channel_unref (out_channel);
	g_io_add_watch (err_channel, G_IO_IN | G_IO_ERR | G_IO_HUP, 
			build_output_cb, 
			info);
	g_io_channel_unref (err_channel);
}

static gboolean
impl_probe (GbfProject *project,
	    const char *path,
	    GError    **error)
{
	/* the good thing about basic backend, is that it can claim anything */
	return TRUE;
}

static void
impl_add_build_callback (GbfProject      *project,
			 GbfBuildCallback callback,
			 gpointer         user_data,
			 GError         **error)
{
	GbfBasicProject *prj = GBF_BASIC_PROJECT (project);
	GbfBasicBuildCallback *cb;

	cb = g_new0 (GbfBasicBuildCallback, 1);
	cb->callback = callback;
	cb->user_data = user_data;

	prj->priv->callbacks = g_list_append (prj->priv->callbacks, cb);
}

#if 0
static int
find_callback (gconstpointer a, gconstpointer b)
{
	UNIMPLEMENTED;
	
	return 0;
}
#endif

static void
impl_remove_build_callback (GbfProject      *project,
			    GbfBuildCallback callback,
			    GError         **error)
{
	UNIMPLEMENTED;
}

static GbfProjectGroup *
impl_get_group (GbfProject *project,
		const char *id,
		GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static GtkWidget *
impl_configure_new_group (GbfProject *project,
			  GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static GtkWidget *
impl_configure_group (GbfProject *project,
		      const char *id,
		      GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static const char *
impl_add_group (GbfProject *project,
		const char *parent_id,
		const char *id,
		GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static void
impl_remove_group (GbfProject *project,
		   const char *id,
		   GError    **error)
{
	UNIMPLEMENTED;
}

static GbfProjectTarget *
impl_get_target (GbfProject *project,
		 const char *id,
		 GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

#if 0
static void
foreach_target (gpointer key, gpointer value, gpointer data)
{
	UNIMPLEMENTED;
}
#endif

static GList *
impl_get_all_targets (GbfProject *project,
		      GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static GtkWidget *
impl_configure_new_target (GbfProject *project,
			   GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static GtkWidget *
impl_configure_target (GbfProject *project,
		       const char *id,
		       GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static const char *
impl_add_target (GbfProject *project,
		 const char *group_id,
		 const char *name,
		 const char *type,
		 GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static void
impl_remove_target (GbfProject *project,
		    const char *id,
		    GError    **error)
{
	UNIMPLEMENTED;
}

static const char *
impl_name_for_type (GbfProject *project,
		    const char *type)
{
	UNIMPLEMENTED;

	return NULL;
}

static GdkPixbuf *
impl_icon_for_type (GbfProject *project,
		    const char *type)
{
	UNIMPLEMENTED;

	return NULL;
}

static GbfProjectTargetSource *
impl_get_source (GbfProject *project,
		 const char *id,
		 GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

#if 0
static void
foreach_source (gpointer key, gpointer value, gpointer data)
{
	UNIMPLEMENTED;
}
#endif

static GList *
impl_get_all_sources (GbfProject *project,
		      GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static GtkWidget *
impl_configure_new_source (GbfProject *project,
			   GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static GtkWidget *
impl_configure_source (GbfProject *project,
		       const char *id,
		       GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

/**
 * impl_add_source:
 * @project: 
 * @target_id: the target ID to where to add the source
 * @uri: the FULL uri to the file.
 * @error: 
 * 
 * Add source implementation.  The uri must have the project root as its parent.
 * 
 * Return value: 
 **/
static const char *
impl_add_source (GbfProject *project,
		 const char *target_id,
		 const char *uri,
		 GError    **error)
{
	UNIMPLEMENTED;

	return NULL;
}

static void
impl_remove_source (GbfProject *project,
		    const char *id,
		    GError    **error)
{
	UNIMPLEMENTED;
}

static void
gbf_basic_project_dispose (GObject *object)
{
	GbfBasicProject        *prj;
	GbfBasicProjectPrivate *priv;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GBF_IS_BASIC_PROJECT (object));

	prj = GBF_BASIC_PROJECT (object);
	
	priv = prj->priv;
	if (priv) {
		g_free(priv->project_dir);
		if (priv->callbacks) {
			g_list_free (priv->callbacks);
			priv->callbacks = NULL;
		}
		g_free (priv);
		prj->priv = NULL;
	}

	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
}

static void
gbf_basic_project_get_property (GObject    *object,
			     guint       prop_id,
			     GValue     *value,
			     GParamSpec *pspec)
{
	GbfBasicProject *project = GBF_BASIC_PROJECT (object);

	switch (prop_id) {
	case PROP_PROJECT_DIR:
		g_value_set_string (value, project->priv->project_dir);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
gbf_basic_project_class_init (GbfBasicProjectClass *klass)
{
	GObjectClass *object_class;
	GbfProjectClass *project_class;

	object_class = G_OBJECT_CLASS (klass);
	project_class = GBF_PROJECT_CLASS (klass);
	parent_class = g_type_class_peek_parent (klass);

	object_class->dispose = gbf_basic_project_dispose;
	object_class->get_property = gbf_basic_project_get_property;

	project_class->load = impl_load;
	project_class->build = impl_build;
	project_class->probe = impl_probe;
	project_class->add_build_callback = impl_add_build_callback;
	project_class->remove_build_callback = impl_remove_build_callback;
	project_class->add_group = impl_add_group;
	project_class->remove_group = impl_remove_group;
	project_class->get_group = impl_get_group;
	project_class->configure_group = impl_configure_group;
	project_class->configure_new_group = impl_configure_new_group;
	project_class->add_target = impl_add_target;
	project_class->remove_target = impl_remove_target;
	project_class->get_target = impl_get_target;
	project_class->get_all_targets = impl_get_all_targets;
	project_class->configure_target = impl_configure_target;
	project_class->configure_new_target = impl_configure_new_target;
	project_class->add_source = impl_add_source;
	project_class->remove_source = impl_remove_source;
	project_class->get_source = impl_get_source;
	project_class->get_all_sources = impl_get_all_sources;
	project_class->configure_source = impl_configure_source;
	project_class->configure_new_source = impl_configure_new_source;
	project_class->name_for_type = impl_name_for_type;
	project_class->icon_for_type = impl_icon_for_type;

	/* default signal handlers */
	project_class->project_updated = NULL;

	g_object_class_install_property 
		(object_class, PROP_PROJECT_DIR,
		 g_param_spec_string ("project-dir", 
				      _("Project directory"),
				      _("Project directory"),
				      "",
				      G_PARAM_READABLE));

}

static void
gbf_basic_project_instance_init (GbfBasicProject *prj)
{
	GbfBasicProjectPrivate *priv;
	
	g_message("gbf_basic_project_instance_init");

	priv = g_new0 (GbfBasicProjectPrivate, 1);
	prj->priv = priv;
}

GBF_BACKEND_BOILERPLATE (GbfBasicProject, gbf_basic_project);

GBF_SIMPLE_BACKEND (GbfBasicProject, gbf_basic_project);
